#!/bin/sh

source ./bfw

mkdir -pv $TEMPFILES || exit 1

distcvs anonymous@glite.cvs.cern.ch:/cvs/glite org.glite.security.voms glite-security-voms_R_1_9_19_2 glite-security-voms-1.9.19.2.tar.bz2
distcvs anonymous@glite.cvs.cern.ch:/cvs/glite org.glite.security.voms-mysql glite-security-voms-mysql_R_3_1_5_1 glite-security-voms-mysql-3.1.5.1.tar.bz2

export GLOBUS_LOCATION=/opt/globus
source /opt/globus/etc/globus-user-env.sh

cd $TEMPFILES
tar xf $DISTFILES/glite-security-voms-1.9.19.2.tar.bz2
cd glite-security-voms-1.9.19.2

# patch
sed 's@\(std::cerr << "VOMS Server for " <<\) contact.vo() \(<< " not known!" << std::endl;\)@\1 (contact.vo().empty() ? contact.nick() : contact.vo()) \2@' -i src/client/vomsclient.cc
sed 's:\(\$(INSTALL)\) \(\$(top_builddir)/src/api/java/vomsjapi.jar\):\1 -m644 \2:' -i src/api/java/Makefile.am
sed 's:validatedChain.clone();:(X509Certificate[])&:' -i src/api/java/org/glite/voms/VOMSValidator.java
sed 's:^mkdir -p \$@LOCATION_ENV@_VAR/log:&/voms:' -i src/install/voms_install_db.in
sed 's:logname=voms.\$voms_vo:logname=$voms_vo:' -i src/install/voms_install_db.in
sed 's:\(logfile=\$@LOCATION_ENV@_VAR/log/\)\(\$logname\):\1voms/\2:' -i src/install/voms_install_db.in
cat <<'END' | patch -p0
--- src/install/voms.start.in	2009-07-16 15:10:40.000000000 +0200
+++ src/install/voms.start.in	2009-09-03 10:10:17.574074641 +0200
@@ -30,9 +30,19 @@
 #	voms servers.
 #
 # chkconfig: 2345 95 05
-# description: VOMS stands for Virtual Organization Membership System.
+# description: Virtual Organization Membership Service
 # processname: voms
 # pidfile: /var/run/voms.pid
+#
+### BEGIN INIT INFO
+# Provides:          voms
+# Required-Start:    $remote_fs $syslog
+# Required-Stop:     $remote_fs $syslog
+# Default-Start:     2 3 4 5
+# Default-Stop:      0 1 6
+# Short-Description: Virtual Organization Membership Service
+# Description:       Virtual Organization Membership Service
+### END INIT INFO
 
 # Source networking configuration and check that networking is up.
 if [ -r /etc/sysconfig/network ] ; then
@@ -62,9 +72,9 @@
 # add lib directory to LD_LIBRARY_PATH
 export LD_LIBRARY_PATH="$@LOCATION_ENV@/lib:$LD_LIBRARY_PATH"
 
-voms=${binpath}/edg-voms
+voms=${binpath}/voms
 if !(test -x ${voms}) ; then
-    echo "Cannot find edg-voms." 
+    echo "Cannot find voms."
     exit 0
 fi
 
@@ -76,6 +86,10 @@
     prog=`basename ${voms}`
     if test -z $1; then
         vos=`ls $@LOCATION_ENV@/etc/voms`
+        if test -z "$vos" ; then
+            echo "No VOMS servers configured."
+            return 0
+        fi
     else
         vos=$1
     fi
@@ -95,7 +109,7 @@
                 read pid < $@VAR_LOCATION_ENV@/lock/subsys/voms.$vo
                 if test "x$pid" != "x"; then
                     if checkpid $pid; then
-                        echo "VOMS ($pid) is already running" && failure
+                        echo "VOMS ($pid) is already running"
                         continue;
                     fi
                 fi
@@ -121,12 +135,17 @@
                 RETGLOB=$(($RETGLOB + 1 ))
             fi
             [ "$RETVAL" -eq 0 ] && success || failure 
+            echo
         fi
     done
     
     return $RETGLOB
 }
 
+if [ -r /etc/init.d/functions ] ; then
+    . /etc/init.d/functions
+else
+
 success()
 {
     echo $1
@@ -162,6 +181,8 @@
     return $rc
 }
 
+fi
+
 checkpid() 
 {
     ps --pid $1 >> /dev/null
@@ -266,8 +287,7 @@
                 fi
             fi
             if test -z "$pids"; then
-                failure; echo
-                RETVAL=1
+                echo "(already stopped)"
                 continue
             fi
             for pid in $pids; do
@@ -284,10 +304,15 @@
 }
 
 function condrestart() {
-    # Stop daemons.
+    # Conditionally restart daemons.
+    RETGLOB=0
     prog=`basename ${voms}`
     if test -z $1; then
         vos=`ls $@LOCATION_ENV@/etc/voms`
+        if test -z "$vos" ; then
+            echo "No VOMS servers configured."
+            return 0
+        fi
     else
         vos=$1
     fi
@@ -297,11 +322,10 @@
             echo -n "Condrestart $prog($vo): "
 #	    pids=$(ps axo pid,args | grep "${voms} --conf $@LOCATION_ENV@/etc/voms/$vo/voms.conf" | grep -v grep | cut -f 1 -d '/')
             pids=`ps -efww | grep "$@LOCATION_ENV@/sbin/edg-voms --conf $@LOCATION_ENV@/etc/voms/$vo/voms.conf" | grep -v grep | awk '{ print $2; }'`
-            if test -z "$pids"; then
-                echo; start $vo
-            else
-                failure; echo
-                RETVAL=1
+            if test -n "$pids"; then
+                $0 restart $vo
+                RETVAL=$?
+                RETGLOB=$((RETGLOB + RETVAL))
             fi
         fi
     done
@@ -311,11 +335,19 @@
 }
 
 function getstatus() {
-    # Stop daemons.
+    # Get status.
     prog=`basename ${voms}`
 
     # get vos
-    vos=${1:-`find $@LOCATION_ENV@/etc/voms/ -type d -printf "%f "`}
+    if test -z $1; then
+        vos=${1:-`find $@LOCATION_ENV@/etc/voms/ -type d -printf "%f "`}
+        if test -z "$vos" ; then
+            echo "No VOMS servers configured."
+            return 0
+        fi
+    else
+        vos=$1
+    fi
 
     value=0
 
@@ -333,7 +365,8 @@
                 value=$[$value < 3 ? 3 : $value]
                 continue
             else
-                echo "(pid $pids) is running..."
+                echo -n "(pid $pids) is running..." | tr '\n' ' '
+                echo
                 value=$[$value < 0 ? 0 : $value]
                 continue
             fi
@@ -357,12 +390,14 @@
         getstatus $2
         RETVAL=$?
         ;;
-    restart)
+    restart | force-reload)
         stop $2
         start $2
         RETVAL=$?
         ;;
-    condrestart)
+    reload)
+        ;;
+    condrestart | try-restart)
         condrestart $2
         RETVAL=$?
         ;;
END
cat <<'END' | patch -p0
--- src/log/log.c 2009-07-16 15:10:40.000000000 +0200
+++ src/log/log.c 2009-09-07 16:02:18.597470396 +0200
@@ -175,7 +175,11 @@
   if (*buffer) {
     while (offset < slen) {
       do {
+#ifdef PIPE_BUF
         readbytes = read(fd, *buffer + offset, (slen - offset > PIPE_BUF ? PIPE_BUF : slen - offset));
+#else
+        readbytes = read(fd, *buffer + offset, (slen - offset > fpathconf(fd, _PC_PIPE_BUF) ? fpathconf(fd, _PC_PIPE_BUF) : slen - offset));
+#endif
       } while (readbytes < 0 && (errno == EINTR
 #ifdef ERESTART
                                    || errno == ERESTART
END
cat <<'END' | patch -p0
diff -ur voms-1.9.11.orig/src/ac/Makefile.am voms-1.9.11/src/ac/Makefile.am
--- src/ac/Makefile.am	2009-07-16 15:10:39.000000000 +0200
+++ src/ac/Makefile.am	2009-09-07 11:06:49.079471423 +0200
@@ -27,6 +27,7 @@
 	-DNOGLOBUS \
 	-I $(top_srcdir)/src/include \
 	-I $(top_builddir)/src/sslutils
+libattributes_nog_la_CXXFLAGS = $(libattributes_nog_la_CFLAGS)
 
 libattributes_nog_la_LIBADD = \
 	$(OPENSSL_LIBS)
--- src/api/ccapi/api_util.cc	2009-07-16 15:10:40.000000000 +0200
+++ src/api/ccapi/api_util.cc	2009-09-07 11:06:49.080471703 +0200
@@ -415,9 +415,9 @@
   /* extract vo name from AC */
   
   AC * ac = (AC *)data;
-  STACK_OF(AC_ATTR) * atts = ac->acinfo->attrib;
+  const STACK_OF(AC_ATTR) * atts = ac->acinfo->attrib;
   int nid = OBJ_txt2nid("idatcap");
-  int pos = X509at_get_attr_by_NID(atts, nid, -1);
+  int pos = X509at_get_attr_by_NID((const STACK_OF(X509_ATTRIBUTE)*)atts, nid, -1);
 
   int nidc = OBJ_txt2nid("certseq");
   STACK_OF(X509_EXTENSION) *exts = ac->acinfo->exts;
--- src/include/acstack.h	2009-07-16 15:10:40.000000000 +0200
+++ src/include/acstack.h	2009-09-07 11:06:49.081472345 +0200
@@ -79,7 +79,7 @@
 #endif
 
 #define DECL_STACK(type) \
-   DECLARE_STACK_OF(type) \
+   PREDECLARE_STACK_OF(type) \
    extern STACK_OF(type) *sk_##type##_new (int (*)(const type * const *, const type * const *)); \
    extern STACK_OF(type) *sk_##type##_new_null (); \
    extern void   sk_##type##_free (STACK_OF(type) *); \
--- src/socklib/Client.cpp	2009-07-16 15:10:41.000000000 +0200
+++ src/socklib/Client.cpp	2009-09-07 11:28:07.210471657 +0200
@@ -336,7 +336,11 @@
 bool 
 GSISocketClient::Open()
 {
+#if OPENSSL_VERSION_NUMBER >= 0x10000000L
+  const SSL_METHOD *meth = NULL;
+#else
   SSL_METHOD *meth = NULL;
+#endif
   int ret = -1, ret2 = -1;
   time_t starttime, curtime;
   long errorcode = 0;
--- src/utils/vomsfake.cc	2009-07-16 15:10:41.000000000 +0200
+++ src/utils/vomsfake.cc	2009-09-07 10:21:52.935470664 +0200
@@ -40,6 +40,7 @@
 #include "listfunc.h"
 #include "credentials.h"
 #include "parsertypes.h"
+#define YYTOKENTYPE
 #include "vomsparser.h"
 
 VOLIST *volist = NULL;
END

# fix bad permissions
find . '(' -name '*.h' -o -name '*.c' -o -name '*.cpp' -o \
           -name '*.cc' -o -name '*.java' ')' -exec chmod a-x {} ';'

# use pdflatex
sed -e 's!^\(USE_PDFLATEX *= *\)NO!\1YES!' -i src/api/ccapi/Makefile.am

# touch to avoid rerunning bison and flex
touch -r src/utils/vomsfake.y src/utils/vomsparser.h
touch -r src/utils/vomsfake.y src/utils/vomsparser.c
touch -r src/utils/vomsfake.y src/utils/lex.yy.c

# rebootstrap
./autogen.sh

# configure
./configure --enable-glite --enable-docs \
            --with-java-home=/usr --with-bc=/usr/share/java/bcprov.jar \
            --with-log4j=/usr/share/java/log4j.jar \
            --with-commons-cli=/usr/share/java/commons-cli.jar \
            --with-commons-lang=/usr/share/java/commons-lang.jar

# make & install
make
make install

cd ..
rm -rf glite-security-voms-1.9.19.2

tar xf $DISTFILES/glite-security-voms-mysql-3.1.5.1.tar.bz2
cd glite-security-voms-mysql-3.1.5.1

./configure --prefix=/opt/glite
make && make install

cd ..
rm -rf glite-security-voms-mysql-3.1.5.1
